import 'dart:convert';

class Purchase {
  String userId;
  String orderId;
  String transactionId;
  DateTime transactionDate;
  String productId;
  String purchaseToken;
  String signatureAndroid;
  String transactionReceipt;

  Purchase({
    this.userId,
    this.orderId,
    this.transactionId,
    this.transactionDate,
    this.productId,
    this.purchaseToken,
    this.signatureAndroid,
    this.transactionReceipt,
  });

  Purchase copyWith({
    String userId,
    String orderId,
    String transactionId,
    DateTime transactionDate,
    String productId,
    String purchaseToken,
    String signatureAndroid,
    String transactionReceipt,
  }) {
    return Purchase(
      userId: userId ?? this.userId,
      orderId: orderId ?? this.orderId,
      transactionId: transactionId ?? this.transactionId,
      transactionDate: transactionDate ?? this.transactionDate,
      productId: productId ?? this.productId,
      purchaseToken: purchaseToken ?? this.purchaseToken,
      signatureAndroid: signatureAndroid ?? this.signatureAndroid,
      transactionReceipt: transactionReceipt ?? this.transactionReceipt,
    );
  }

  Map<String, dynamic> toMap({forEncoding = true}) {
    return {
      'user_id': userId,
      'order_id': orderId,
      'transaction_id': transactionId,
      'transaction_date': forEncoding
          ? transactionDate.millisecondsSinceEpoch
          : transactionDate,
      'product_id': productId,
      'purchase_token': purchaseToken,
      'signature_android': signatureAndroid,
      'transaction_receipt': transactionReceipt,
    };
  }

  factory Purchase.fromMap(Map<String, dynamic> map) {
    if (map == null) return null;

    try {
      return Purchase(
        userId: map['user_id'],
        orderId: map['order_id'],
        transactionId: map['transaction_id'],
        transactionDate: map['transaction_date'] is int
            ? DateTime.fromMillisecondsSinceEpoch(map['transaction_date'])
            : map['transaction_date'],
        productId: map['product_id'],
        purchaseToken: map['purchase_token'],
        signatureAndroid: map['signature_android'],
        transactionReceipt: map['transaction_receipt'],
      );
    } catch (e) {
      print(e);
      return null;
    }
  }

  String toJson() => json.encode(toMap());

  factory Purchase.fromJson(String source) =>
      Purchase.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Purchase(userId: $userId, orderId: $orderId, transactionId: $transactionId, transactionDate: $transactionDate, productId: $productId, purchaseToken: $purchaseToken, signatureAndroid: $signatureAndroid, transactionReceipt: $transactionReceipt)';
  }

  @override
  bool operator ==(Object o) {
    if (identical(this, o)) return true;

    return o is Purchase &&
        o.userId == userId &&
        o.orderId == orderId &&
        o.transactionId == transactionId &&
        o.transactionDate == transactionDate &&
        o.productId == productId &&
        o.purchaseToken == purchaseToken &&
        o.signatureAndroid == signatureAndroid &&
        o.transactionReceipt == transactionReceipt;
  }

  @override
  int get hashCode {
    return userId.hashCode ^
        orderId.hashCode ^
        transactionId.hashCode ^
        transactionDate.hashCode ^
        productId.hashCode ^
        purchaseToken.hashCode ^
        signatureAndroid.hashCode ^
        transactionReceipt.hashCode;
  }
}
