import 'package:bmcommons/purchase/purchase.dart';
import 'package:bmcommons/purchase/purchases_firestore_repository.dart';
import 'package:bmcommons/purchase/purchases_local_repository.dart';
import 'package:flutter/material.dart';
import 'package:flutter_inapp_purchase/flutter_inapp_purchase.dart';
import 'package:flutter_inapp_purchase/modules.dart';

class SavePurchaseUsecase {
  final PurchasesFirestoreRepository purchasesFirestoreRepository;
  final PurchasesLocalRepository purchasesLocalRepository;
  final FlutterInappPurchase flutterInappPurchase;

  SavePurchaseUsecase(
      this.purchasesFirestoreRepository, this.purchasesLocalRepository, this.flutterInappPurchase);

  Future savePurchase({@required PurchasedItem productItem, @required String userId}) async {
    try {
      var purchase = Purchase.fromMap({
        'user_id': userId,
        'order_id': productItem.orderId,
        'transaction_id': productItem.transactionId,
        'transaction_date': productItem.transactionDate,
        'product_id': productItem.productId,
        'purchase_token': productItem.purchaseToken,
        'signature_android': productItem.signatureAndroid,
        'transaction_receipt': productItem.transactionReceipt,
      });

      await purchasesFirestoreRepository.savePurchase(
        productItem.transactionId ?? productItem.orderId,
        purchase: purchase.toMap(forEncoding: false),
      );
      await purchasesLocalRepository.savePurchase(
        purchase: purchase.toJson(),
      );
      await flutterInappPurchase.finishTransaction(productItem);
    } catch (e) {
      return null;
    }
  }
}
